/***********
 This module is provided as an example of creating a Node PACKAGE, which could be published to NPM.

 Any modules within subfolders should be made into packages *create a package.json, e.g. by running 'npm init' in the directory).
 Otherwise NPM may clean up subfolders when running npm commands.
 **********/

const omnis_calls = require('omnis_calls');
const http = require('http');
const fs = require('fs');
const os = require('os');
const path = require('path');

/**
 * Implement all of your publicly accessible methods in methodMap.
 */
const methodMap = {
    spike: function (obj, response) {
        omnis_calls.sendResponse({"foo": "bar"}, response);
        return true;
    },

	processLogos: function() {
		//get path
        function getPath(){
            var isWin = process.platform === "win32";
            if(isWin)
                return os.homedir()+"\\Library\\LOCALAPPDATA%\\Suran\\logos\\CDM+\\"
            else
                return os.homedir()+"/Library/Application Support/Suran/logos/CDM+/"
        }

        //download logos
        function getFile(logoID){
            let file = fs.createWriteStream(getPath()+"LogoCDM_"+logoID+"_545x204_x2.png");
            request = http.get("http://download.suran.com/logos/cdm/LogoCDM_"+logoID+"_545x204_x2.png", function(response) {
                response.pipe(file);
            });
        }

        //delete logos
        function deleteFile(logoID){
            var path = getPath()+"LogoCDM_"+logoID+"_545x204_x2.png";
            try {
                fs.unlinkSync(path)
              } catch(err) {
                console.error(err)
              }
        }

        //downloads a new manifest
        function getManifest(){
            let file2 = fs.createWriteStream(getPath()+"manifest.json");
            request = http.get("http://download.suran.com/logos/cdm/catalog.json", function(response) {
                response.pipe(file2);
                file2.on('finish', function() {
                    logoActions();
                    file2.close();
                });
            });
        }
        getManifest();

        //get array of currently downloaded logos
        function findFilesInDir(startPath){
            var results = [];
            if (!fs.existsSync(startPath)){
                return;
            }
            var files=fs.readdirSync(startPath);
            for(var i=0;i<files.length;i++){
                if(files[i] != "manifest.json" && files[i] != ".DS_Store"){
                    temp = files[i];
                    temp = temp.split("_");
                    results.push(temp[1]);
                }
            }
            return results;
        }

        //convert manifest to list
        function convertManifest(){
            var manifestLogoID = new Array;
            manifest = fs.readFileSync(getPath()+'manifest.json', 'utf8');
            var temp = manifest.split(",");
            for(var i = 0; i < temp.length; i+=5){
                temp[i] = temp[i].replace('[','');
                temp[i] = temp[i].replace('[','');
                manifestLogoID.push(temp[i]);
            }
            return manifestLogoID;
        }

        //compare manifest with downloaded logos
        function getDeleteList(manifestLogos, currentLogos){
            var deleteList = new Array;
            for(var i = 0; i<currentLogos.length; i++){
                if(!(manifestLogos.includes(currentLogos[i]))){
                    deleteList.push(currentLogos[i]);
                }
            }
            return deleteList;
        }
        function getDownloadList(manifestLogos, currentLogos){
            var downloadList = new Array;
            for(var i = 0; i<manifestLogos.length; i++){
                if(!(currentLogos.includes(manifestLogos[i]))){
                    downloadList.push(manifestLogos[i]);
                }   
            }
            return downloadList
        }

        function logoActions(){
            var manifestLogos = convertManifest();
            var currentLogos = findFilesInDir(getPath());
            var logoIDList = getDownloadList(manifestLogos, currentLogos);
            for(i = 0; i<logoIDList.length; i++){
                getFile(logoIDList[i]);
            }
            logoIDList = getDeleteList(manifestLogos, currentLogos);
            for(i = 0; i<logoIDList.length; i++){
                deleteFile(logoIDList[i]);
            }
        }
	}
};


module.exports = {
	call: function (method, param, response) { // The only requirement of an Omnis module is that it implement this function.

		if (methodMap[method]) {
			const result = methodMap[method](param, response);
			return true;
		}
		else {
			throw Error("Method '" + method + "' does not exist");
		}

	}
};